package MetaTF;

import java.io.*;
import MetaTF.Generator.Java.CodeGenerator;

public class MetaTFtool {
  private static final String cvsheader = "$Header: /home/cur/rej/cvs/MetaTF/src/MetaTF/MetaTFtool.java,v 1.4 2000/12/18 17:05:48 rej Exp $";

    public static void main(String args[]) {

      MetaTF.Parser.MetaTFParser parser=null;
      MetaTF.Parser.DTD symtab=null;
      String dtdFile=null;
      String outputFile=null;
      char language = 'J';  // J=Java, C=C

      switch(args.length) {

        case 0:
          //System.out.println("MetaTF Parser Version 1.0.:  Reading from standard input . . .");
          dtdFile = "<stdin>";
          parser = new MetaTF.Parser.MetaTFParser(System.in);
          break;
          
        case 1:
            //System.out.println("Java Parser Version 1.0:  Reading from file " + args[0] + " . . .");
            try {
                dtdFile = args[0];
                parser = new MetaTF.Parser.MetaTFParser(new FileInputStream(args[0]));
            } catch (java.io.FileNotFoundException e) {
                System.out.println("MetaTF Parser Version 1.0:  File " + args[0] + " not found.");
                return;
            }
            break;

         case 2:
             if(args[0].equalsIgnoreCase("-C"))
                 language = 'C';
             else if(args[0].equalsIgnoreCase("-J"))
                 language = 'J';
             else
                 usage();
             outputFile = args[1];
             //System.out.println("MetaTF Parser Version 1.0.:  Reading from standard input . . .");
             dtdFile = "<stdin>";
             parser = new MetaTF.Parser.MetaTFParser(System.in);
             break;

         case 3:
             if(args[0].equalsIgnoreCase("-C"))
                 language = 'C';
             else if(args[0].equalsIgnoreCase("-J"))
                 language = 'J';
             else
                 usage();
             outputFile = args[1];
             try {
                 dtdFile = args[2];
                 parser = new MetaTF.Parser.MetaTFParser(new FileInputStream(args[2]));
             } catch (java.io.FileNotFoundException e) {
                 System.out.println("MetaTF Parser Version 1.0:  File " + args[2] + " not found.");
                 return;
             }
             break;

         default:
             usage();
             return;
      }       

      MetaTF.Parser.ASTDTD ast=null;

      try {
          ast = parser.DTD();
          //System.out.println("MetaTF Parser Version 1.0:  DTD parsed successfully.");
          //ast.dump(">");
      } catch (MetaTF.Parser.ParseException e) {
          System.err.println("MetaTF Parser Version 1.0.2:  Encountered errors during parse.");
          e.printStackTrace();
          System.exit(1);
      }

      try {
          //System.err.println("MetaTFtool: Building symbol table...");
          symtab = MetaTF.Parser.DTD.instance();
          MetaTF.Parser.STBuilder stb = new MetaTF.Parser.STBuilder();
          ast.jjtAccept(stb, symtab);
          //System.err.println("MetaTFtool: Symbol table built.");
          //System.err.println(symtab.dump("? "));
          // Get a list of fields in the order they appear
          ast.jjtAccept(new MetaTF.Parser.FieldLister(),symtab);
      } catch (MetaTF.Parser.DTDException e) {
          System.err.println("MetaTFtool semantic error:\n"+e);
          System.exit(1);
      }

      // Now generate the code.
      if(outputFile!=null) {
          int p = dtdFile.lastIndexOf(File.separatorChar);
	  dtdFile = dtdFile.substring(p+1);
          if(language=='J') {
              System.err.println("MetaTFTool: Generating code into "+outputFile+"...");
              CodeGenerator gen = new CodeGenerator(dtdFile, outputFile);
              gen.generate(symtab, null);
              //System.err.println("MetaTFtool: Finished generating code");
          } else {
              System.err.println("MetaTFtool: Sorry, generator for language not implemented");
              System.exit(1);
          }
      }
    }
 

    private static void usage() {
        System.out.println("MetaTFtool usage:");
        System.out.println("  java MetaTFtool [-C|J class] < inputfile");
        System.out.println("  java MetaTFtool [-C|J class] inputfile");
        System.exit(1);
    }

}
