/*
    $Id$
    Date        Author          Changes
    Nov 20 2000 Richard Jones   Created
    Nov 21 2000 Richard Jones   Changed import structure

    Copyright (c) 2000, Richard Jones
    All rights reserved.
 */
 
package MetaTF.Runtime;

import java.io.*;
import MetaTF.MetaTFConstants;
import MetaTF.BadRecordException;

/**
 * AsciiReader implements LowLevelReader
 *
 * @version rel-1-2 ($Revision: 1.1 $, $Date: 2000/12/15 16:35:32 $),
 * @author Richard Jones
**/
public class AsciiReader implements LowLevelReader {

    private PushbackInputStream rdr;

    public AsciiReader(InputStream di) {
    	rdr = new PushbackInputStream(di);
    }

    // Avoid Java Unicode bumpf!
    private static boolean isdigit(int c) {
	return ('0' <= c && c <= '9');
    }

    private static boolean isxdigit(int c) {
	return ( ('0' <= c && c <= '9') || ('a' <= c && c <= 'f') || ('A' <= c && c <= 'F'));
    }

    private static boolean islower(int c) {
        return ('a' <= c && c <= 'z');
    }

    private static int toupper(int c) {
        return c + 'A' - 'a';
    }

    private static long readLong(int c, PushbackInputStream rdr) throws IOException, BadRecordException {
	int base = 10;
	if(c == '0') {
	    base = 8;
	    c = rdr.read();
	    if(c == 'x') {
		int c1 = rdr.read();
		if(isxdigit(c1)) {
		    c = c1;
		    base = 16;
		} else {
    	            throw new BadRecordException("Number expected but found "+(char)c1);
		    //rdr.unread();
		}
	    }
	}
	int value;
	int result = 0;
	while(isxdigit(c) && ((value = isdigit(c) ? c-'0' : 
		              (islower(c) ? toupper(c) : c) - 'A' + 10) < base) ) {
	    result = result*base + value;
	    c = rdr.read();
	}
	rdr.unread(c);
	return result;
    }

    public int readTag() throws BadRecordException, IOException {
    	int c;
    	do {    	// discard leading white space etc
    	    c = rdr.read();
    	} while(Character.isWhitespace((char)c));
    	if(c==-1)
    	    throw new EOFException("End of file");
    	    
    	if(c==MetaTFConstants.Metadata)
    	    return MetaTFConstants.MetadataTag;
    	else if(c==MetaTFConstants.Section)
    	    return MetaTFConstants.SectionTag;
    	else if(c==MetaTFConstants.Comment)
    	    return MetaTFConstants.CommentTag;
	
	// it should be a positive integer
	return (int) readLong(c, rdr);
    }

    public long readNum() throws BadRecordException, IOException {
    	int c;
    	do {    	// discard leading white space etc
    	    c = rdr.read();
    	} while(Character.isWhitespace((char)c));
    	if(c==-1)
    	    throw new EOFException("End of file");
    	    
    	int sign = 1;
    	if(c=='-') {
    	    sign = -1;
    	    c = rdr.read();
    	    if(c==-1)
    	        throw new EOFException("End of file");
    	}
	return sign * readLong(c, rdr);
    }

    public String readAny() throws BadRecordException, IOException {
        int c;
    	do {    	// discard leading white space etc
    	    c = rdr.read();
    	} while(Character.isWhitespace((char)c));
    	if(c==-1)
    	    throw new EOFException("End of file");
    	StringBuffer buf = new StringBuffer();
    	do {
    	    buf.append((char)c);
    	    c = rdr.read();
    	} while(!Character.isWhitespace((char)c) && c!=-1);
    	return buf.toString();
    }
    
    public String readString() throws BadRecordException, IOException {
    	int c;
    	do {    	// discard leading white space etc
    	    c = rdr.read();
    	} while(Character.isWhitespace((char)c));
    	if(c==-1)
    	    throw new EOFException("End of file");
    	    
    	StringBuffer buf = new StringBuffer();
    	if(c != '\"')
    	    throw new BadRecordException("\" expected but found "+(char)c);
    	c = rdr.read();
    	do {
            if(c==-1)
    	        throw new EOFException("End of file found while reading string");
            if(c=='\\') {
               c = rdr.read();
               if(c==-1)
    	            throw new EOFException("End of file found while reading string");
               switch(c) {
                 case 'n':  buf.append("\n"); break;
                 case 't':  buf.append("\t"); break;
                 case 'b':  buf.append("\b"); break;
                 case 'r':  buf.append("\r"); break;
                 case 'f':  buf.append("\f"); break;
                 case '\\': buf.append("\\"); break;
                 case '\'': buf.append("\'"); break;
                 case '\"': buf.append("\""); break;                    	
                 default:
                     if('0'>=c || c>='9') {
                        buf.append(c);
                     } else {
                         int ch = c - '0';
                         c = rdr.read();
                         if(c==-1)
                	          throw new EOFException("End of file found while reading string");
                         if('0' <= c && c <= '7') {
                             ch = ch*8 + c - '0';
                             int prev = c;
                             c = rdr.read();
                             if(c==-1)
                	             throw new EOFException("End of file found while reading string");
        	                 if(prev <= '3' && '0' <= c && c <= '7') {
        	                     ch = ch*8 + c - '0';
        	                 }
                         }
                         buf.append((char)ch);
                         break;
                     }
                   }
            } else if(c=='"') {
                return buf.toString();
            } else {
                buf.append((char)c);
            }
            
            c = rdr.read();
            if(c==-1)
    	        throw new EOFException("End of file found while reading string");
        } while(true);
    }

    public String readln() throws BadRecordException, IOException {
        int c;
        StringBuffer buf = new StringBuffer();
        c = rdr.read();
        while(c!=-1 && c!='\n') {
            buf.append((char)c);
            c = rdr.read();
        }
        return buf.toString();
    }

    public static void main(String[] args) {
        int tag;
        long num;
        String str;
        AsciiReader rdr = new AsciiReader(System.in);
        try {
            while(true) {
                try {
                    tag = rdr.readTag();
                    num = rdr.readNum();
                    str = rdr.readString();

                    System.err.println(tag+" "+num+" "+str);
                 } catch (BadRecordException e) {
                     System.err.println(e.getMessage());
                 }
            };
        } catch(EOFException e) {
        } catch(IOException e) {
            System.err.println(e.getMessage());
        }
    }
}
