/*
    $File$
    Date        Author          Changes
    Nov 30 2000 Richard Jones   Created
    
    Copyright (c) 2000, Richard Jones
    All rights reserved.
 */
 
package MetaTF.Runtime;

import MetaTF.MetaTFConstants;
import MetaTF.BadRecordException;
import MetaTF.Panic;
import java.util.List;

/**
 * A TraceField representing a field with numeric type
 *
 * @version rel-1-2 ($Revision: 1.2 $, $Date: 2001/03/29 15:53:24 $),
 * @author Richard Jones
**/
public class NumTraceField extends TraceField {

    /** The first argument of the encoding **/
   private long encoding1;

    /** The second argument of the encoding **/
    private long encoding2;                            
                                                                             
    /**
     * Allocate a new object representing a encoding of a field of numeric type
     *
     * @param n the name of the field
     * @param it the encoding
     * @param i1 the first encoding argument
     * @param i2 the first encoding argument
    **/
    public NumTraceField(String n, int it, long i1, long i2) {               
        super(n);                                                            
        try {                                                                
            setInterpretationType(it);                                       
            encoding1 = i1;                                            
            encoding2 = i2;                                            
        } catch(BadRecordException e) {                                        
            Panic.panic("Cannot create NumTraceField for "+n);                   
        }                                                                    
    }                                                                        

    /**
     * Allocate a new object representing a encoding of a field of numeric type
     *
     * @param n the name of the field
     * @param it the encoding
     * @param i1 the first encoding argument
    **/
    public NumTraceField(String n, int it, long i1) {               
        this(n, it, i1, 0);                                                  
    }                                                                        

    /**
     * Allocate a new object representing a encoding of a field of numeric type
     *
     * @param n the name of the field
     * @param it the encoding
    **/
    public NumTraceField(String n, int it) {               
        this(n, it, 0, 0);                                                  
    }                                                                        

    void setInterpretationType(int i) throws BadRecordException {         
        if(i==MetaTFConstants.SectionOffset || i==MetaTFConstants.SectionStride) 
            throw new BadRecordException("cannot set a sectionOffset or sectionStride encoding in a NumTraceField"); 
        encodingType = i;                                              
    }                                                                        

    void setValue(long v) throws BadRecordException {                 
        encoding1 = v;                                                 
    }                                                                        

    void setValue(String s) throws BadRecordException {               
        throw new BadRecordException("string value given to numeric field "+s);  
    }                                                                        

    void setValue2(long v) throws BadRecordException {                
        encoding2 = v;                                                 
    }                                                                        

    long getValue() {                                                 
        if(encodingType != MetaTFConstants.None)                       
            return encoding1;                                          
        else {                                                               
            Panic.panic("Cannot getValue of field with encoding none");    
            return 0; //NEVER REACHED                                        
        }                                                                    
    }                                                                        

    long getValue2() {                                                
        if(!(encodingType==MetaTFConstants.Stride || encodingType==MetaTFConstants.SectionStride)) { 
            Panic.panic("Cannot getValue2 of fields with non-stride encodings"); 
            return 0; //NEVER REACHED                                        
        } else                                                               
            return encoding2;                                          
    }                                                                        
}                                                                            

